"""Approvaltests module."""

from .version import version_number
from .approvals import *
from .asserts import *
from .combination_approvals import *

__author__ = "Chris Lucian; Llewellyn Falco; Jim Counts; Tim Preece"
__version__ = version_number
