"""Approvaltests.reporters module."""

from .clipboard_reporter import *
from .default_reporter_factory import *
from .diff_reporter import *
from .first_working_reporter import *
from .generic_diff_reporter import *
from .generic_diff_reporter_config import *
from .generic_diff_reporter_factory import *
from .multi_reporter import *
from .python_native_reporter import *
from .received_file_launcher_reporter import *
from .reporter_that_automatically_approves import *
from .report_all_to_clipboard import *
from .report_by_creating_diff_file import *
